@@ -38,6 +38,35 @@ WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
 #define HTMLELEM_THIS(iface) DEFINE_THIS(HTMLElement, HTMLElement, iface)
 
+HRESULT create_nselem(HTMLDocumentNode *doc, const WCHAR *tag, nsIDOMHTMLElement **ret)
+{
+    nsIDOMElement *nselem;
+    nsAString tag_str;
+    nsresult nsres;
+
+    if(!doc->nsdoc) {
+        WARN("NULL nsdoc\n");
+        return E_UNEXPECTED;
+    }
+
+    nsAString_Init(&tag_str, tag);
+    nsres = nsIDOMDocument_CreateElement(doc->nsdoc, &tag_str, &nselem);
+    nsAString_Finish(&tag_str);
+    if(NS_FAILED(nsres)) {
+        ERR("CreateElement failed: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    nsres = nsIDOMElement_QueryInterface(nselem, &IID_nsIDOMHTMLElement, (void**)ret);
+    nsIDOMElement_Release(nselem);
+    if(NS_FAILED(nsres)) {
+        ERR("Could not get nsIDOMHTMLElement iface: %08x\n", nsres);
+        return E_FAIL;
+    }
+
+    return S_OK;
+}
+
 #define HTMLELEM_NODE_THIS(iface) DEFINE_THIS2(HTMLElement, node, iface)
 
 static HRESULT WINAPI HTMLElement_QueryInterface(IHTMLElement *iface,
@@ -96,44 +125,26 @@ static HRESULT WINAPI HTMLElement_setAttribute(IHTMLElement *iface, BSTR strAttr
                                                VARIANT AttributeValue, LONG lFlags)
 {
     HTMLElement *This = HTMLELEM_THIS(iface);
-    nsAString attr_str;
-    nsAString value_str;
-    nsresult nsres;
     HRESULT hres;
-    VARIANT AttributeValueChanged;
+    DISPID dispid, dispidNamed = DISPID_PROPERTYPUT;
+    DISPPARAMS dispParams;
+    EXCEPINFO excep;
 
-    WARN("(%p)->(%s . %08x)\n", This, debugstr_w(strAttributeName), lFlags);
+    TRACE("(%p)->(%s . %08x)\n", This, debugstr_w(strAttributeName), lFlags);
 
-    if(!This->nselem) {
-        FIXME("NULL nselem\n");
-        return E_NOTIMPL;
-    }
-
-    VariantInit(&AttributeValueChanged);
-
-    hres = VariantChangeType(&AttributeValueChanged, &AttributeValue, 0, VT_BSTR);
-    if (FAILED(hres)) {
-        WARN("couldn't convert input attribute value %d to VT_BSTR\n", V_VT(&AttributeValue));
+    hres = IDispatchEx_GetDispID(DISPATCHEX(&This->node.dispex), strAttributeName,
+            fdexNameCaseInsensitive | fdexNameEnsure, &dispid);
+    if(FAILED(hres))
         return hres;
-    }
 
-    nsAString_Init(&attr_str, strAttributeName);
-    nsAString_Init(&value_str, V_BSTR(&AttributeValueChanged));
-
-    TRACE("setting %s to %s\n", debugstr_w(strAttributeName),
-        debugstr_w(V_BSTR(&AttributeValueChanged)));
-
-    nsres = nsIDOMHTMLElement_SetAttribute(This->nselem, &attr_str, &value_str);
-    nsAString_Finish(&attr_str);
-    nsAString_Finish(&value_str);
-
-    if(NS_SUCCEEDED(nsres)) {
-        hres = S_OK;
-    }else {
-        ERR("SetAttribute failed: %08x\n", nsres);
-        hres = E_FAIL;
-    }
+    dispParams.cArgs = 1;
+    dispParams.cNamedArgs = 1;
+    dispParams.rgdispidNamedArgs = &dispidNamed;
+    dispParams.rgvarg = &AttributeValue;
 
+    hres = IDispatchEx_InvokeEx(DISPATCHEX(&This->node.dispex), dispid,
+            LOCALE_SYSTEM_DEFAULT, DISPATCH_PROPERTYPUT, &dispParams,
+            NULL, &excep, NULL);
     return hres;
 }
 
@@ -141,59 +152,28 @@ static HRESULT WINAPI HTMLElement_getAttribute(IHTMLElement *iface, BSTR strAttr
                                                LONG lFlags, VARIANT *AttributeValue)
 {
     HTMLElement *This = HTMLELEM_THIS(iface);
-    nsAString attr_str;
-    nsAString value_str;
-    const PRUnichar *value;
-    nsresult nsres;
-    HRESULT hres = S_OK;
+    DISPID dispid;
+    HRESULT hres;
+    DISPPARAMS dispParams = {NULL, NULL, 0, 0};
+    EXCEPINFO excep;
 
-    WARN("(%p)->(%s %08x %p)\n", This, debugstr_w(strAttributeName), lFlags, AttributeValue);
+    TRACE("(%p)->(%s %08x %p)\n", This, debugstr_w(strAttributeName), lFlags, AttributeValue);
 
-    if(!This->nselem) {
-        FIXME("NULL nselem\n");
+    hres = IDispatchEx_GetDispID(DISPATCHEX(&This->node.dispex), strAttributeName,
+            fdexNameCaseInsensitive, &dispid);
+    if(hres == DISP_E_UNKNOWNNAME) {
         V_VT(AttributeValue) = VT_NULL;
         return S_OK;
     }
 
-    V_VT(AttributeValue) = VT_NULL;
-
-    nsAString_Init(&attr_str, strAttributeName);
-    nsAString_Init(&value_str, NULL);
-
-    nsres = nsIDOMHTMLElement_GetAttribute(This->nselem, &attr_str, &value_str);
-    nsAString_Finish(&attr_str);
-
-    if(NS_SUCCEEDED(nsres)) {
-        static const WCHAR wszSRC[] = {'s','r','c',0};
-        nsAString_GetData(&value_str, &value);
-        if(!strcmpiW(strAttributeName, wszSRC))
-        {
-            WCHAR buffer[256];
-            DWORD len;
-            BSTR bstrBaseUrl;
-            hres = IHTMLDocument2_get_URL(HTMLDOC(This->node.doc), &bstrBaseUrl);
-            if(SUCCEEDED(hres)) {
-                hres = CoInternetCombineUrl(bstrBaseUrl, value,
-                                            URL_ESCAPE_SPACES_ONLY|URL_DONT_ESCAPE_EXTRA_INFO,
-                                            buffer, sizeof(buffer)/sizeof(WCHAR), &len, 0);
-                SysFreeString(bstrBaseUrl);
-                if(SUCCEEDED(hres)) {
-                    V_VT(AttributeValue) = VT_BSTR;
-                    V_BSTR(AttributeValue) = SysAllocString(buffer);
-                    TRACE("attr_value=%s\n", debugstr_w(V_BSTR(AttributeValue)));
-                }
-            }
-        }else if(*value) {
-            V_VT(AttributeValue) = VT_BSTR;
-            V_BSTR(AttributeValue) = SysAllocString(value);
-            TRACE("attr_value=%s\n", debugstr_w(V_BSTR(AttributeValue)));
-        }
-    }else {
-        ERR("GetAttribute failed: %08x\n", nsres);
-        hres = E_FAIL;
+    if(FAILED(hres)) {
+        V_VT(AttributeValue) = VT_NULL;
+        return hres;
     }
 
-    nsAString_Finish(&value_str);
+    hres = IDispatchEx_InvokeEx(DISPATCHEX(&This->node.dispex), dispid,
+            LOCALE_SYSTEM_DEFAULT, DISPATCH_PROPERTYGET, &dispParams,
+            AttributeValue, &excep, NULL);
 
     return hres;
 }
@@ -431,15 +411,19 @@ static HRESULT WINAPI HTMLElement_get_onclick(IHTMLElement *iface, VARIANT *p)
 static HRESULT WINAPI HTMLElement_put_ondblclick(IHTMLElement *iface, VARIANT v)
 {
     HTMLElement *This = HTMLELEM_THIS(iface);
-    FIXME("(%p)->()\n", This);
-    return E_NOTIMPL;
+
+    FIXME("(%p)->(%s)\n", This, debugstr_variant(&v));
+
+    return set_node_event(&This->node, EVENTID_DBLCLICK, &v);
 }
 
 static HRESULT WINAPI HTMLElement_get_ondblclick(IHTMLElement *iface, VARIANT *p)
 {
     HTMLElement *This = HTMLELEM_THIS(iface);
-    FIXME("(%p)->(%p)\n", This, p);
-    return E_NOTIMPL;
+
+    TRACE("(%p)->(%p)\n", This, p);
+
+    return get_node_event(&This->node, EVENTID_DBLCLICK, p);
 }
 
 static HRESULT WINAPI HTMLElement_put_onkeydown(IHTMLElement *iface, VARIANT v)
@@ -585,7 +569,7 @@ static HRESULT WINAPI HTMLElement_get_document(IHTMLElement *iface, IDispatch **
     if(!p)
         return E_POINTER;
 
-    *p = (IDispatch*)HTMLDOC(This->node.doc);
+    *p = (IDispatch*)HTMLDOC(&This->node.doc->basedoc);
     IDispatch_AddRef(*p);
 
     return S_OK;
@@ -1148,8 +1132,10 @@ static HRESULT WINAPI HTMLElement_get_isTextEdit(IHTMLElement *iface, VARIANT_BO
 static HRESULT WINAPI HTMLElement_click(IHTMLElement *iface)
 {
     HTMLElement *This = HTMLELEM_THIS(iface);
-    FIXME("(%p)\n", This);
-    return E_NOTIMPL;
+
+    TRACE("(%p)\n", This);
+
+    return call_event(&This->node, EVENTID_CLICK);
 }
 
 static HRESULT WINAPI HTMLElement_get_filters(IHTMLElement *iface,
@@ -1503,7 +1489,7 @@ static dispex_static_data_t HTMLElement_dispex = {
     HTMLElement_iface_tids
 };
 
-void HTMLElement_Init(HTMLElement *This)
+void HTMLElement_Init(HTMLElement *This, dispex_static_data_t *dispex_data)
 {
     This->lpHTMLElementVtbl = &HTMLElementVtbl;
 
@@ -1512,11 +1498,10 @@ void HTMLElement_Init(HTMLElement *This)
     HTMLElement2_Init(This);
     HTMLElement3_Init(This);
 
-    if(!This->node.dispex.data)
-        init_dispex(&This->node.dispex, (IUnknown*)HTMLELEM(This), &HTMLElement_dispex);
+    init_dispex(&This->node.dispex, (IUnknown*)HTMLELEM(This), dispex_data ? dispex_data : &HTMLElement_dispex);
 }
 
-HTMLElement *HTMLElement_Create(HTMLDocument *doc, nsIDOMNode *nsnode, BOOL use_generic)
+HTMLElement *HTMLElement_Create(HTMLDocumentNode *doc, nsIDOMNode *nsnode, BOOL use_generic)
 {
     nsIDOMHTMLElement *nselem;
     HTMLElement *ret = NULL;
@@ -1572,7 +1557,7 @@ HTMLElement *HTMLElement_Create(HTMLDocument *doc, nsIDOMNode *nsnode, BOOL use_
 
     if(!ret) {
         ret = heap_alloc_zero(sizeof(HTMLElement));
-        HTMLElement_Init(ret);
+        HTMLElement_Init(ret, NULL);
         ret->node.vtbl = &HTMLElementImplVtbl;
     }
 
