@@ -43,7 +43,7 @@ typedef struct {
     LONG ref;
 
     nsIDOMRange *nsrange;
-    HTMLDocument *doc;
+    HTMLDocumentNode *doc;
 
     struct list entry;
 } HTMLTxtRange;
@@ -72,7 +72,7 @@ typedef enum {
     RU_TEXTEDIT
 } range_unit_t;
 
-static HTMLTxtRange *get_range_object(HTMLDocument *doc, IHTMLTxtRange *iface)
+static HTMLTxtRange *get_range_object(HTMLDocumentNode *doc, IHTMLTxtRange *iface)
 {
     HTMLTxtRange *iter;
 
@@ -1211,14 +1211,15 @@ static HRESULT WINAPI HTMLTxtRange_duplicate(IHTMLTxtRange *iface, IHTMLTxtRange
 {
     HTMLTxtRange *This = HTMLTXTRANGE_THIS(iface);
     nsIDOMRange *nsrange = NULL;
+    HRESULT hres;
 
     TRACE("(%p)->(%p)\n", This, Duplicate);
 
     nsIDOMRange_CloneRange(This->nsrange, &nsrange);
-    *Duplicate = HTMLTxtRange_Create(This->doc, nsrange);
+    hres = HTMLTxtRange_Create(This->doc, nsrange, Duplicate);
     nsIDOMRange_Release(nsrange);
 
-    return S_OK;
+    return hres;
 }
 
 static HRESULT WINAPI HTMLTxtRange_inRange(IHTMLTxtRange *iface, IHTMLTxtRange *Range,
@@ -1545,23 +1546,20 @@ static HRESULT WINAPI HTMLTxtRange_moveEnd(IHTMLTxtRange *iface, BSTR Unit,
 static HRESULT WINAPI HTMLTxtRange_select(IHTMLTxtRange *iface)
 {
     HTMLTxtRange *This = HTMLTXTRANGE_THIS(iface);
+    nsISelection *nsselection;
+    nsresult nsres;
 
     TRACE("(%p)\n", This);
 
-    if(This->doc->nscontainer) {
-        nsIDOMWindow *dom_window = NULL;
-        nsISelection *nsselection;
-
-        nsIWebBrowser_GetContentDOMWindow(This->doc->nscontainer->webbrowser, &dom_window);
-        nsIDOMWindow_GetSelection(dom_window, &nsselection);
-        nsIDOMWindow_Release(dom_window);
-
-        nsISelection_RemoveAllRanges(nsselection);
-        nsISelection_AddRange(nsselection, This->nsrange);
-
-        nsISelection_Release(nsselection);
+    nsres = nsIDOMWindow_GetSelection(This->doc->basedoc.window->nswindow, &nsselection);
+    if(NS_FAILED(nsres)) {
+        ERR("GetSelection failed: %08x\n", nsres);
+        return E_FAIL;
     }
 
+    nsISelection_RemoveAllRanges(nsselection);
+    nsISelection_AddRange(nsselection, This->nsrange);
+    nsISelection_Release(nsselection);
     return S_OK;
 }
 
@@ -1780,10 +1778,9 @@ static HRESULT WINAPI RangeCommandTarget_QueryStatus(IOleCommandTarget *iface, c
 
 static HRESULT exec_indent(HTMLTxtRange *This, VARIANT *in, VARIANT *out)
 {
+    nsIDOMHTMLElement *blockquote_elem, *p_elem;
     nsIDOMDocumentFragment *fragment;
-    nsIDOMElement *blockquote_elem, *p_elem;
     nsIDOMNode *tmp;
-    nsAString tag_str;
 
     static const PRUnichar blockquoteW[] = {'B','L','O','C','K','Q','U','O','T','E',0};
     static const PRUnichar pW[] = {'P',0};
@@ -1795,13 +1792,8 @@ static HRESULT exec_indent(HTMLTxtRange *This, VARIANT *in, VARIANT *out)
         return E_NOTIMPL;
     }
 
-    nsAString_Init(&tag_str, blockquoteW);
-    nsIDOMHTMLDocument_CreateElement(This->doc->nsdoc, &tag_str, &blockquote_elem);
-    nsAString_Finish(&tag_str);
-
-    nsAString_Init(&tag_str, pW);
-    nsIDOMDocument_CreateElement(This->doc->nsdoc, &tag_str, &p_elem);
-    nsAString_Finish(&tag_str);
+    create_nselem(This->doc, blockquoteW, &blockquote_elem);
+    create_nselem(This->doc, pW, &p_elem);
 
     nsIDOMRange_ExtractContents(This->nsrange, &fragment);
     nsIDOMElement_AppendChild(p_elem, (nsIDOMNode*)fragment, &tmp);
@@ -1850,9 +1842,13 @@ static const IOleCommandTargetVtbl OleCommandTargetVtbl = {
     RangeCommandTarget_Exec
 };
 
-IHTMLTxtRange *HTMLTxtRange_Create(HTMLDocument *doc, nsIDOMRange *nsrange)
+HRESULT HTMLTxtRange_Create(HTMLDocumentNode *doc, nsIDOMRange *nsrange, IHTMLTxtRange **p)
 {
-    HTMLTxtRange *ret = heap_alloc(sizeof(HTMLTxtRange));
+    HTMLTxtRange *ret;
+
+    ret = heap_alloc(sizeof(HTMLTxtRange));
+    if(!ret)
+        return E_OUTOFMEMORY;
 
     ret->lpHTMLTxtRangeVtbl = &HTMLTxtRangeVtbl;
     ret->lpOleCommandTargetVtbl = &OleCommandTargetVtbl;
@@ -1865,10 +1861,11 @@ IHTMLTxtRange *HTMLTxtRange_Create(HTMLDocument *doc, nsIDOMRange *nsrange)
     ret->doc = doc;
     list_add_head(&doc->range_list, &ret->entry);
 
-    return HTMLTXTRANGE(ret);
+    *p = HTMLTXTRANGE(ret);
+    return S_OK;
 }
 
-void detach_ranges(HTMLDocument *This)
+void detach_ranges(HTMLDocumentNode *This)
 {
     HTMLTxtRange *iter;
 
