@@ -323,7 +323,7 @@ static inline void call_event_handler( Display *display, XEvent *event )
 /***********************************************************************
  *           process_events
  */
-static int process_events( Display *display, Bool (*filter)(), ULONG_PTR arg )
+static int process_events( Display *display, Bool (*filter)(Display*, XEvent*,XPointer), ULONG_PTR arg )
 {
     XEvent event, prev_event;
     int count = 0;
@@ -463,6 +463,7 @@ static inline BOOL can_activate_window( HWND hwnd )
     if (!(style & WS_VISIBLE)) return FALSE;
     if ((style & (WS_POPUP|WS_CHILD)) == WS_CHILD) return FALSE;
     if (style & WS_MINIMIZE) return FALSE;
+    if (GetWindowLongW( hwnd, GWL_EXSTYLE ) & WS_EX_NOACTIVATE) return FALSE;
     if (hwnd == GetDesktopWindow()) return FALSE;
     return !(style & WS_DISABLED);
 }
@@ -511,6 +512,7 @@ static void handle_wm_protocols( HWND hwnd, XClientMessageEvent *event )
         if (IsWindowEnabled(hwnd))
         {
             HMENU hSysMenu;
+            POINT pt;
 
             if (GetClassLongW(hwnd, GCL_STYLE) & CS_NOCLOSE) return;
             hSysMenu = GetSystemMenu(hwnd, FALSE);
@@ -524,7 +526,7 @@ static void handle_wm_protocols( HWND hwnd, XClientMessageEvent *event )
             {
                 LRESULT ma = SendMessageW( hwnd, WM_MOUSEACTIVATE,
                                            (WPARAM)GetAncestor( hwnd, GA_ROOT ),
-                                           MAKELONG(HTCLOSE,WM_LBUTTONDOWN) );
+                                           MAKELPARAM( HTCLOSE, WM_NCLBUTTONDOWN ) );
                 switch(ma)
                 {
                     case MA_NOACTIVATEANDEAT:
@@ -541,7 +543,10 @@ static void handle_wm_protocols( HWND hwnd, XClientMessageEvent *event )
                         break;
                 }
             }
-            PostMessageW( hwnd, WM_X11DRV_DELETE_WINDOW, 0, 0 );
+            /* Simulate clicking the caption Close button */
+            GetCursorPos( &pt );
+            PostMessageW( hwnd, WM_NCLBUTTONDOWN, HTCLOSE, MAKELPARAM( pt.x, pt.y ) );
+            PostMessageW( hwnd, WM_LBUTTONUP, HTCLOSE, MAKELPARAM( pt.x, pt.y ) );
         }
     }
     else if (protocol == x11drv_atom(WM_TAKE_FOCUS))
